# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["SubdomainGetResponse"]


class SubdomainGetResponse(BaseModel):
    subdomain: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from ..._models import BaseModel

__all__ = ["AutoOriginTLSKexGetResponse"]


class AutoOriginTLSKexGetResponse(BaseModel):
    id: str

    enabled: bool
    """Whether Auto-Origin TLS KEX selection is enabled for the zone."""

    modified_on: datetime
    """Last time this setting was modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .order_get_response import OrderGetResponse as OrderGetResponse
from .order_update_params import OrderUpdateParams as OrderUpdateParams
from .order_update_response import OrderUpdateResponse as OrderUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from ...shared.response_info import ResponseInfo

__all__ = ["PrefixBindingDeleteResponse"]


class PrefixBindingDeleteResponse(BaseModel):
    messages: List[ResponseInfo]

    success: bool

    errors: Optional[List[ResponseInfo]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.email_auth import dmarc_report_edit_params
from ...types.email_auth.dmarc_report_get_response import DMARCReportGetResponse
from ...types.email_auth.dmarc_report_edit_response import DMARCReportEditResponse

__all__ = ["DMARCReportsResource", "AsyncDMARCReportsResource"]


class DMARCReportsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DMARCReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DMARCReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DMARCReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DMARCReportsResourceWithStreamingResponse(self)

    def edit(
        self,
        *,
        zone_id: str,
        enabled: Optional[bool] | Omit = omit,
        skip_wizard: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DMARCReportEditResponse]:
        """Updates the DMARC report configuration for a zone.

        At least one of `enabled` or
        `skip_wizard` must be provided. When enabling, the handler will ensure the DMARC
        RUA record exists in DNS.

        Args:
          zone_id: Identifier.

          enabled: Enable or disable DMARC reports for this zone

          skip_wizard: Skip the DMARC setup wizard

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._patch(
            path_template("/zones/{zone_id}/email/auth/dmarc-reports", zone_id=zone_id),
            body=maybe_transform(
                {
                    "enabled": enabled,
                    "skip_wizard": skip_wizard,
                },
                dmarc_report_edit_params.DMARCReportEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DMARCReportEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DMARCReportEditResponse]], ResultWrapper[DMARCReportEditResponse]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DMARCReportGetResponse]:
        """Retrieves the current DMARC report configuration and status for a zone.

        Returns
        the RUA prefix, enabled status, approved sources, and DNS records.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/email/auth/dmarc-reports", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DMARCReportGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DMARCReportGetResponse]], ResultWrapper[DMARCReportGetResponse]),
        )


class AsyncDMARCReportsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDMARCReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDMARCReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDMARCReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDMARCReportsResourceWithStreamingResponse(self)

    async def edit(
        self,
        *,
        zone_id: str,
        enabled: Optional[bool] | Omit = omit,
        skip_wizard: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DMARCReportEditResponse]:
        """Updates the DMARC report configuration for a zone.

        At least one of `enabled` or
        `skip_wizard` must be provided. When enabling, the handler will ensure the DMARC
        RUA record exists in DNS.

        Args:
          zone_id: Identifier.

          enabled: Enable or disable DMARC reports for this zone

          skip_wizard: Skip the DMARC setup wizard

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._patch(
            path_template("/zones/{zone_id}/email/auth/dmarc-reports", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "enabled": enabled,
                    "skip_wizard": skip_wizard,
                },
                dmarc_report_edit_params.DMARCReportEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DMARCReportEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DMARCReportEditResponse]], ResultWrapper[DMARCReportEditResponse]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DMARCReportGetResponse]:
        """Retrieves the current DMARC report configuration and status for a zone.

        Returns
        the RUA prefix, enabled status, approved sources, and DNS records.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/email/auth/dmarc-reports", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DMARCReportGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DMARCReportGetResponse]], ResultWrapper[DMARCReportGetResponse]),
        )


class DMARCReportsResourceWithRawResponse:
    def __init__(self, dmarc_reports: DMARCReportsResource) -> None:
        self._dmarc_reports = dmarc_reports

        self.edit = to_raw_response_wrapper(
            dmarc_reports.edit,
        )
        self.get = to_raw_response_wrapper(
            dmarc_reports.get,
        )


class AsyncDMARCReportsResourceWithRawResponse:
    def __init__(self, dmarc_reports: AsyncDMARCReportsResource) -> None:
        self._dmarc_reports = dmarc_reports

        self.edit = async_to_raw_response_wrapper(
            dmarc_reports.edit,
        )
        self.get = async_to_raw_response_wrapper(
            dmarc_reports.get,
        )


class DMARCReportsResourceWithStreamingResponse:
    def __init__(self, dmarc_reports: DMARCReportsResource) -> None:
        self._dmarc_reports = dmarc_reports

        self.edit = to_streamed_response_wrapper(
            dmarc_reports.edit,
        )
        self.get = to_streamed_response_wrapper(
            dmarc_reports.get,
        )


class AsyncDMARCReportsResourceWithStreamingResponse:
    def __init__(self, dmarc_reports: AsyncDMARCReportsResource) -> None:
        self._dmarc_reports = dmarc_reports

        self.edit = async_to_streamed_response_wrapper(
            dmarc_reports.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            dmarc_reports.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = [
    "DMARCReportEditResponse",
    "ApprovedSource",
    "Records",
    "RecordsBimiRecord",
    "RecordsCnamedkimRecord",
    "RecordsCnamedmarcRecord",
    "RecordsCnamespfRecord",
    "RecordsDKIMRecord",
    "RecordsDMARCRecord",
    "RecordsSPFRecord",
]


class ApprovedSource(BaseModel):
    """A single approved sending source"""

    created: Optional[datetime] = None
    """Deprecated, use created_at"""

    created_at: Optional[datetime] = None
    """Creation timestamp"""

    domain: Optional[str] = None
    """The source domain"""

    ips: Optional[List[str]] = None
    """Resolved IP addresses from SPF"""

    modified: Optional[datetime] = None
    """Deprecated, use modified_at"""

    modified_at: Optional[datetime] = None
    """Last modification timestamp"""

    name: Optional[str] = None
    """Source name (typically same as domain)"""

    slug: Optional[str] = None
    """URL-friendly identifier"""

    tag: Optional[str] = None
    """Source UUID"""


class RecordsBimiRecord(BaseModel):
    """Summary of a single DNS record"""

    id: Optional[str] = None
    """DNS record ID"""

    content: Optional[str] = None
    """Record content"""

    name: Optional[str] = None
    """DNS record name"""

    ttl: Optional[int] = None
    """Time to live in seconds"""

    type: Optional[str] = None
    """Record type"""


class RecordsCnamedkimRecord(BaseModel):
    """Summary of a single DNS record"""

    id: Optional[str] = None
    """DNS record ID"""

    content: Optional[str] = None
    """Record content"""

    name: Optional[str] = None
    """DNS record name"""

    ttl: Optional[int] = None
    """Time to live in seconds"""

    type: Optional[str] = None
    """Record type"""


class RecordsCnamedmarcRecord(BaseModel):
    """Summary of a single DNS record"""

    id: Optional[str] = None
    """DNS record ID"""

    content: Optional[str] = None
    """Record content"""

    name: Optional[str] = None
    """DNS record name"""

    ttl: Optional[int] = None
    """Time to live in seconds"""

    type: Optional[str] = None
    """Record type"""


class RecordsCnamespfRecord(BaseModel):
    """Summary of a single DNS record"""

    id: Optional[str] = None
    """DNS record ID"""

    content: Optional[str] = None
    """Record content"""

    name: Optional[str] = None
    """DNS record name"""

    ttl: Optional[int] = None
    """Time to live in seconds"""

    type: Optional[str] = None
    """Record type"""


class RecordsDKIMRecord(BaseModel):
    """Summary of a single DNS record"""

    id: Optional[str] = None
    """DNS record ID"""

    content: Optional[str] = None
    """Record content"""

    name: Optional[str] = None
    """DNS record name"""

    ttl: Optional[int] = None
    """Time to live in seconds"""

    type: Optional[str] = None
    """Record type"""


class RecordsDMARCRecord(BaseModel):
    """Summary of a single DNS record"""

    id: Optional[str] = None
    """DNS record ID"""

    content: Optional[str] = None
    """Record content"""

    name: Optional[str] = None
    """DNS record name"""

    ttl: Optional[int] = None
    """Time to live in seconds"""

    type: Optional[str] = None
    """Record type"""


class RecordsSPFRecord(BaseModel):
    """Summary of a single DNS record"""

    id: Optional[str] = None
    """DNS record ID"""

    content: Optional[str] = None
    """Record content"""

    name: Optional[str] = None
    """DNS record name"""

    ttl: Optional[int] = None
    """Time to live in seconds"""

    type: Optional[str] = None
    """Record type"""


class Records(BaseModel):
    """Live DNS records for the zone, grouped by type"""

    bimi_records: Optional[List[RecordsBimiRecord]] = None
    """BIMI TXT records"""

    cname_dkim_records: Optional[List[RecordsCnamedkimRecord]] = None
    """CNAME records for DKIM"""

    cname_dmarc_records: Optional[List[RecordsCnamedmarcRecord]] = None
    """CNAME records at \\__dmarc (problematic)"""

    cname_spf_records: Optional[List[RecordsCnamespfRecord]] = None
    """CNAME records for SPF"""

    dkim_records: Optional[List[RecordsDKIMRecord]] = None
    """DKIM TXT records"""

    dmarc_records: Optional[List[RecordsDMARCRecord]] = None
    """DMARC TXT records"""

    spf_records: Optional[List[RecordsSPFRecord]] = None
    """SPF TXT records"""


class DMARCReportEditResponse(BaseModel):
    """Response for GET/PATCH /dmarc-reports"""

    approved_sources: Optional[List[ApprovedSource]] = None
    """List of approved sending sources (omitted when empty)"""

    created: Optional[datetime] = None
    """Deprecated, use created_at"""

    created_at: Optional[datetime] = None
    """Creation timestamp"""

    enabled: Optional[bool] = None
    """Whether DMARC reports are enabled"""

    modified: Optional[datetime] = None
    """Deprecated, use modified_at"""

    modified_at: Optional[datetime] = None
    """Last modification timestamp"""

    records: Optional[Records] = None
    """Live DNS records for the zone, grouped by type"""

    rua_prefix: Optional[str] = None
    """Prefix for DMARC RUA addresses (32-char hex string)"""

    skip_wizard: Optional[bool] = None
    """Whether to skip the setup wizard"""

    status: Optional[
        Literal["missing-dmarc-report", "multiple-dmarc-reports", "missing-dmarc-rua", "cname-on-dmarc-record"]
    ] = None
    """DMARC configuration status"""

    tag: Optional[str] = None
    """Use `zone_id` instead"""

    zone_id: Optional[str] = None
    """Zone identifier"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .query import (
    QueryResource,
    AsyncQueryResource,
    QueryResourceWithRawResponse,
    AsyncQueryResourceWithRawResponse,
    QueryResourceWithStreamingResponse,
    AsyncQueryResourceWithStreamingResponse,
)
from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from .log_explorer import (
    LogExplorerResource,
    AsyncLogExplorerResource,
    LogExplorerResourceWithRawResponse,
    AsyncLogExplorerResourceWithRawResponse,
    LogExplorerResourceWithStreamingResponse,
    AsyncLogExplorerResourceWithStreamingResponse,
)

__all__ = [
    "QueryResource",
    "AsyncQueryResource",
    "QueryResourceWithRawResponse",
    "AsyncQueryResourceWithRawResponse",
    "QueryResourceWithStreamingResponse",
    "AsyncQueryResourceWithStreamingResponse",
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
    "LogExplorerResource",
    "AsyncLogExplorerResource",
    "LogExplorerResourceWithRawResponse",
    "AsyncLogExplorerResourceWithRawResponse",
    "LogExplorerResourceWithStreamingResponse",
    "AsyncLogExplorerResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["UsageGetResponse", "UsageGetResponseItem"]


class UsageGetResponseItem(BaseModel):
    """
    A single cost and usage record for a metered product within a specific charge period, aligned with the FinOps FOCUS v1.3 specification.
    """

    billing_account_id: str = FieldInfo(alias="BillingAccountId")
    """Public identifier of the Cloudflare account (account tag)."""

    billing_account_name: str = FieldInfo(alias="BillingAccountName")
    """Display name of the Cloudflare account."""

    charge_category: Literal["Usage"] = FieldInfo(alias="ChargeCategory")
    """
    Highest-level classification of a charge based on the nature of how it gets
    billed. Currently only "Usage" is supported.
    """

    charge_description: str = FieldInfo(alias="ChargeDescription")
    """Self-contained summary of the charge's purpose and price."""

    charge_frequency: Literal["Usage-Based"] = FieldInfo(alias="ChargeFrequency")
    """Indicates how often a charge occurs. Currently only "Usage-Based" is supported."""

    charge_period_end: datetime = FieldInfo(alias="ChargePeriodEnd")
    """Exclusive end of the time interval during which the usage was consumed."""

    charge_period_start: datetime = FieldInfo(alias="ChargePeriodStart")
    """Inclusive start of the time interval during which the usage was consumed."""

    consumed_quantity: float = FieldInfo(alias="ConsumedQuantity")
    """Measured usage amount within the charge period.

    Reflects raw metered consumption before pricing transformations.
    """

    consumed_unit: str = FieldInfo(alias="ConsumedUnit")
    """
    Unit of measure for the consumed quantity (e.g., "GB", "Requests",
    "vCPU-Hours").
    """

    host_provider_name: str = FieldInfo(alias="HostProviderName")
    """Name of the entity providing the underlying infrastructure or platform."""

    invoice_issuer_name: str = FieldInfo(alias="InvoiceIssuerName")
    """Name of the entity responsible for invoicing for the services consumed."""

    service_provider_name: str = FieldInfo(alias="ServiceProviderName")
    """Name of the entity that made the services available for purchase."""

    x_billable_metric_name: str = FieldInfo(alias="x_BillableMetricName")
    """The display name of the billable metric.

    Cloudflare extension; replaces FOCUS SkuMeter.
    """

    billed_cost: Optional[float] = FieldInfo(alias="BilledCost", default=None)
    """
    A charge serving as the basis for invoicing, inclusive of all reduced rates and
    discounts while excluding the amortization of upfront charges (one-time or
    recurring).
    """

    billing_currency: Optional[str] = FieldInfo(alias="BillingCurrency", default=None)
    """Currency that a charge was billed in (ISO 4217)."""

    billing_period_end: Optional[datetime] = FieldInfo(alias="BillingPeriodEnd", default=None)
    """Exclusive end of the billing cycle that contains this usage record."""

    billing_period_start: Optional[datetime] = FieldInfo(alias="BillingPeriodStart", default=None)
    """Inclusive start of the billing cycle that contains this usage record."""

    charge_class: Optional[Literal["Correction"]] = FieldInfo(alias="ChargeClass", default=None)
    """
    Indicates whether the row represents a correction to one or more charges
    invoiced in a previous billing period.
    """

    contracted_cost: Optional[float] = FieldInfo(alias="ContractedCost", default=None)
    """
    Cost calculated by multiplying ContractedUnitPrice and the corresponding
    PricingQuantity.
    """

    contracted_unit_price: Optional[float] = FieldInfo(alias="ContractedUnitPrice", default=None)
    """
    The agreed-upon unit price for a single PricingUnit of the associated billable
    metric, inclusive of negotiated discounts, if present, while excluding any other
    discounts.
    """

    effective_cost: Optional[float] = FieldInfo(alias="EffectiveCost", default=None)
    """
    The amortized cost of the charge after applying all reduced rates, discounts,
    and the applicable portion of relevant, prepaid purchases (one-time or
    recurring) that covered the charge.
    """

    list_cost: Optional[float] = FieldInfo(alias="ListCost", default=None)
    """
    Cost calculated by multiplying ListUnitPrice and the corresponding
    PricingQuantity.
    """

    list_unit_price: Optional[float] = FieldInfo(alias="ListUnitPrice", default=None)
    """
    Suggested provider-published unit price for a single PricingUnit of the
    associated billable metric, exclusive of any discounts.
    """

    pricing_quantity: Optional[float] = FieldInfo(alias="PricingQuantity", default=None)
    """Volume of a given service used or purchased, based on the PricingUnit."""

    pricing_unit: Optional[str] = FieldInfo(alias="PricingUnit", default=None)
    """
    Provider-specified measurement unit for determining unit prices, indicating how
    the provider rates measured usage after applying pricing rules like block
    pricing.
    """

    region_id: Optional[str] = FieldInfo(alias="RegionId", default=None)
    """
    Provider-assigned identifier for an isolated geographic area where a service is
    provided.
    """

    region_name: Optional[str] = FieldInfo(alias="RegionName", default=None)
    """Name of an isolated geographic area where a service is provided."""

    sub_account_id: Optional[str] = FieldInfo(alias="SubAccountId", default=None)
    """Unique identifier assigned to a grouping of services.

    For Cloudflare, this is the subscription or contract ID.
    """

    sub_account_name: Optional[str] = FieldInfo(alias="SubAccountName", default=None)
    """Name assigned to a grouping of services.

    For Cloudflare, this is the subscription or contract display name.
    """

    x_billable_metric_id: Optional[str] = FieldInfo(alias="x_BillableMetricId", default=None)
    """The unique identifier for the billable metric in the Cloudflare catalog.

    Cloudflare extension; replaces FOCUS SkuId.
    """

    x_product_family_name: Optional[str] = FieldInfo(alias="x_ProductFamilyName", default=None)
    """The product family the charge belongs to (e.g., "R2", "Workers").

    Cloudflare extension; replaces FOCUS ServiceName.
    """

    x_zone_id: Optional[str] = FieldInfo(alias="x_ZoneId", default=None)
    """The identifier for the Cloudflare zone (zone tag). Cloudflare extension."""

    x_zone_name: Optional[str] = FieldInfo(alias="x_ZoneName", default=None)
    """The display name of the Cloudflare zone. Cloudflare extension."""


UsageGetResponse: TypeAlias = List[UsageGetResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["IdPFederationGrantCreateParams"]


class IdPFederationGrantCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    idp_id: Required[str]
    """UID of the identity provider to federate.

    Must be an existing identity provider in this account. One-time pin and
    Cloudflare-managed identity providers cannot be federated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["AppGetResponse"]


class AppGetResponse(BaseModel):
    id: str

    created_at: str

    name: str

    updated_at: str

    updated_by: str
    """
    Email of the actor who last modified the app, or `edge-gateway` for
    gateway-authenticated changes.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["CustomCsrCreateParams"]


class CustomCsrCreateParams(TypedDict, total=False):
    common_name: Required[str]
    """The common name (domain) for the CSR. Must be at most 64 characters."""

    country: Required[str]
    """Two-letter ISO 3166-1 alpha-2 country code."""

    locality: Required[str]
    """City or locality name."""

    organization: Required[str]
    """Organization name."""

    sans: Required[SequenceNotStr[str]]
    """Subject Alternative Names for the CSR. At least one SAN is required."""

    state: Required[str]
    """State or province name."""

    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    description: str
    """Optional description for the CSR."""

    key_type: Literal["rsa2048", "p256v1"]
    """Key algorithm to use for the CSR. Defaults to rsa2048 if not specified."""

    name: str
    """Human-readable name for the CSR."""

    organizational_unit: str
    """Organizational unit name."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["ServerUpdateParams", "UpdatedPrompt", "UpdatedTool"]


class ServerUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    auth_credentials: str

    description: Optional[str]

    is_shared_oauth_callback_enabled: bool
    """
    When true, the gateway worker uses the shared Cloudflare-owned OAuth callback
    endpoint as the redirect_uri for upstream on-behalf OAuth, instead of the
    customer portal hostname. Defaults to false (off); opt in per server by setting
    true. Effective behavior is gated by the gateway worker's per-env rollout mode
    KV key.
    """

    name: str

    secure_web_gateway: bool
    """Route outbound traffic to this MCP server through Zero Trust Secure Web Gateway"""

    updated_prompts: Iterable[UpdatedPrompt]

    updated_tools: Iterable[UpdatedTool]


class UpdatedPrompt(TypedDict, total=False):
    name: Required[str]

    alias: str

    description: str

    enabled: bool


class UpdatedTool(TypedDict, total=False):
    name: Required[str]

    alias: str

    description: str

    enabled: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.flagship.apps import evaluate_get_params
from ....types.flagship.apps.evaluate_get_response import EvaluateGetResponse

__all__ = ["EvaluateResource", "AsyncEvaluateResource"]


class EvaluateResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EvaluateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EvaluateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EvaluateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EvaluateResourceWithStreamingResponse(self)

    def get(
        self,
        app_id: str,
        *,
        account_id: str,
        flag_key: str,
        targeting_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluateGetResponse:
        """Evaluates a flag against the provided context.

        Pass context attributes as query
        parameters; boolean and numeric strings are coerced automatically. For
        low-latency in-Worker evaluation, prefer the Flagship binding over this
        endpoint.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          flag_key: The flag key to evaluate.

          targeting_key: Context targeting key (per OpenFeature spec); used for percentage rollout
              bucketing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/flagship/apps/{app_id}/evaluate", account_id=account_id, app_id=app_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "flag_key": flag_key,
                        "targeting_key": targeting_key,
                    },
                    evaluate_get_params.EvaluateGetParams,
                ),
            ),
            cast_to=EvaluateGetResponse,
        )


class AsyncEvaluateResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEvaluateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEvaluateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEvaluateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEvaluateResourceWithStreamingResponse(self)

    async def get(
        self,
        app_id: str,
        *,
        account_id: str,
        flag_key: str,
        targeting_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluateGetResponse:
        """Evaluates a flag against the provided context.

        Pass context attributes as query
        parameters; boolean and numeric strings are coerced automatically. For
        low-latency in-Worker evaluation, prefer the Flagship binding over this
        endpoint.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          flag_key: The flag key to evaluate.

          targeting_key: Context targeting key (per OpenFeature spec); used for percentage rollout
              bucketing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/flagship/apps/{app_id}/evaluate", account_id=account_id, app_id=app_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "flag_key": flag_key,
                        "targeting_key": targeting_key,
                    },
                    evaluate_get_params.EvaluateGetParams,
                ),
            ),
            cast_to=EvaluateGetResponse,
        )


class EvaluateResourceWithRawResponse:
    def __init__(self, evaluate: EvaluateResource) -> None:
        self._evaluate = evaluate

        self.get = to_raw_response_wrapper(
            evaluate.get,
        )


class AsyncEvaluateResourceWithRawResponse:
    def __init__(self, evaluate: AsyncEvaluateResource) -> None:
        self._evaluate = evaluate

        self.get = async_to_raw_response_wrapper(
            evaluate.get,
        )


class EvaluateResourceWithStreamingResponse:
    def __init__(self, evaluate: EvaluateResource) -> None:
        self._evaluate = evaluate

        self.get = to_streamed_response_wrapper(
            evaluate.get,
        )


class AsyncEvaluateResourceWithStreamingResponse:
    def __init__(self, evaluate: AsyncEvaluateResource) -> None:
        self._evaluate = evaluate

        self.get = async_to_streamed_response_wrapper(
            evaluate.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .cf1_site_location_param import Cf1SiteLocationParam

__all__ = ["Cf1SiteUpdateParams"]


class Cf1SiteUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    description: str
    """A human-provided description of the CF1 Site."""

    location: Cf1SiteLocationParam

    name: str
    """
    A human-provided name describing the CF1 Site that should be unique within the
    account.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["StatusGetResponse"]


class StatusGetResponse(BaseModel):
    enabled: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .order import (
    OrderResource,
    AsyncOrderResource,
    OrderResourceWithRawResponse,
    AsyncOrderResourceWithRawResponse,
    OrderResourceWithStreamingResponse,
    AsyncOrderResourceWithStreamingResponse,
)
from .levels import (
    LevelsResource,
    AsyncLevelsResource,
    LevelsResourceWithRawResponse,
    AsyncLevelsResourceWithRawResponse,
    LevelsResourceWithStreamingResponse,
    AsyncLevelsResourceWithStreamingResponse,
)

__all__ = [
    "OrderResource",
    "AsyncOrderResource",
    "OrderResourceWithRawResponse",
    "AsyncOrderResourceWithRawResponse",
    "OrderResourceWithStreamingResponse",
    "AsyncOrderResourceWithStreamingResponse",
    "LevelsResource",
    "AsyncLevelsResource",
    "LevelsResourceWithRawResponse",
    "AsyncLevelsResourceWithRawResponse",
    "LevelsResourceWithStreamingResponse",
    "AsyncLevelsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["DataClassCreateParams", "SensitivityLevel"]


class DataClassCreateParams(TypedDict, total=False):
    account_id: Required[str]

    data_tags: Required[SequenceNotStr[str]]

    expression: Required[str]

    name: Required[str]

    sensitivity_levels: Required[Iterable[SensitivityLevel]]

    description: Optional[str]


class SensitivityLevel(TypedDict, total=False):
    """
    A reference pairing a sensitivity group with a specific level within that group.
    """

    group_id: Required[str]

    level_id: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PrefixEditParams"]


class PrefixEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier of a Cloudflare account."""

    description: Required[str]
    """Description of the prefix."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["CustomUpdateResponse"]


class CustomUpdateResponse(BaseModel):
    domain: str
    """Domain name of the affected custom domain."""

    ciphers: Optional[List[str]] = None
    """An allowlist of ciphers for TLS termination.

    These ciphers must be in the BoringSSL format.
    """

    enabled: Optional[bool] = None
    """Whether this bucket is publicly accessible at the specified custom domain."""

    min_tls: Optional[Literal["1.0", "1.1", "1.2", "1.3"]] = FieldInfo(alias="minTLS", default=None)
    """Minimum TLS Version the custom domain will accept for incoming connections.

    If not set, defaults to 1.0.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["SummaryCompromisedParams"]


class SummaryCompromisedParams(TypedDict, total=False):
    bot_class: Annotated[List[Literal["LIKELY_AUTOMATED", "LIKELY_HUMAN"]], PropertyInfo(alias="botClass")]
    """Filters results by bot class.

    Refer to
    [Bot classes](https://developers.cloudflare.com/radar/concepts/bot-classes/).
    """

    date_end: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateEnd", format="iso8601")]
    """End of the date range (inclusive)."""

    date_range: Annotated[SequenceNotStr[str], PropertyInfo(alias="dateRange")]
    """Filters results by date range.

    For example, use `7d` and `7dcontrol` to compare this week with the previous
    week. Use this parameter or set specific start and end dates (`dateStart` and
    `dateEnd` parameters).
    """

    date_start: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateStart", format="iso8601")]
    """Start of the date range."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    name: SequenceNotStr[str]
    """Array of names used to label the series in the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["TelemetryLiveTailResponse"]


class TelemetryLiveTailResponse(BaseModel):
    ws_url: str = FieldInfo(alias="wsUrl")
    """WebSocket URL clients connect to in order to stream live tail events."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.load_balancers.preview_get_response import PreviewGetResponse

__all__ = ["PreviewsResource", "AsyncPreviewsResource"]


class PreviewsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PreviewsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PreviewsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PreviewsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PreviewsResourceWithStreamingResponse(self)

    def get(
        self,
        preview_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PreviewGetResponse:
        """
        Get the result of a previous preview operation using the provided preview_id.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not preview_id:
            raise ValueError(f"Expected a non-empty value for `preview_id` but received {preview_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/load_balancers/preview/{preview_id}",
                account_id=account_id,
                preview_id=preview_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PreviewGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[PreviewGetResponse], ResultWrapper[PreviewGetResponse]),
        )


class AsyncPreviewsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPreviewsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPreviewsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPreviewsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPreviewsResourceWithStreamingResponse(self)

    async def get(
        self,
        preview_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PreviewGetResponse:
        """
        Get the result of a previous preview operation using the provided preview_id.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not preview_id:
            raise ValueError(f"Expected a non-empty value for `preview_id` but received {preview_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/load_balancers/preview/{preview_id}",
                account_id=account_id,
                preview_id=preview_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PreviewGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[PreviewGetResponse], ResultWrapper[PreviewGetResponse]),
        )


class PreviewsResourceWithRawResponse:
    def __init__(self, previews: PreviewsResource) -> None:
        self._previews = previews

        self.get = to_raw_response_wrapper(
            previews.get,
        )


class AsyncPreviewsResourceWithRawResponse:
    def __init__(self, previews: AsyncPreviewsResource) -> None:
        self._previews = previews

        self.get = async_to_raw_response_wrapper(
            previews.get,
        )


class PreviewsResourceWithStreamingResponse:
    def __init__(self, previews: PreviewsResource) -> None:
        self._previews = previews

        self.get = to_streamed_response_wrapper(
            previews.get,
        )


class AsyncPreviewsResourceWithStreamingResponse:
    def __init__(self, previews: AsyncPreviewsResource) -> None:
        self._previews = previews

        self.get = async_to_streamed_response_wrapper(
            previews.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from .schema_data import SchemaData

__all__ = ["SchemaHTTP", "TargetPolicy"]


class TargetPolicy(BaseModel):
    id: str
    """The id of the DEX rule."""

    default: Optional[bool] = None
    """Whether the DEX rule is the account default."""

    name: Optional[str] = None
    """The name of the DEX rule."""


class SchemaHTTP(BaseModel):
    data: SchemaData
    """
    The configuration object which contains the details for the WARP client to
    conduct the test.
    """

    enabled: bool
    """Determines whether or not the test is active."""

    interval: str
    """How often the test will run."""

    name: str
    """The name of the DEX test. Must be unique."""

    description: Optional[str] = None
    """Additional details about the test."""

    target_policies: Optional[List[TargetPolicy]] = None
    """DEX rules targeted by this test"""

    targeted: Optional[bool] = None

    test_id: Optional[str] = None
    """The unique identifier for the test."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .token_rotate_params import TokenRotateParams as TokenRotateParams
from .token_rotate_response import TokenRotateResponse as TokenRotateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .query import (
    QueryResource,
    AsyncQueryResource,
    QueryResourceWithRawResponse,
    AsyncQueryResourceWithRawResponse,
    QueryResourceWithStreamingResponse,
    AsyncQueryResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .datasets.datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)

__all__ = ["LogExplorerResource", "AsyncLogExplorerResource"]


class LogExplorerResource(SyncAPIResource):
    @cached_property
    def query(self) -> QueryResource:
        return QueryResource(self._client)

    @cached_property
    def datasets(self) -> DatasetsResource:
        return DatasetsResource(self._client)

    @cached_property
    def with_raw_response(self) -> LogExplorerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LogExplorerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LogExplorerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LogExplorerResourceWithStreamingResponse(self)


class AsyncLogExplorerResource(AsyncAPIResource):
    @cached_property
    def query(self) -> AsyncQueryResource:
        return AsyncQueryResource(self._client)

    @cached_property
    def datasets(self) -> AsyncDatasetsResource:
        return AsyncDatasetsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLogExplorerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLogExplorerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLogExplorerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLogExplorerResourceWithStreamingResponse(self)


class LogExplorerResourceWithRawResponse:
    def __init__(self, log_explorer: LogExplorerResource) -> None:
        self._log_explorer = log_explorer

    @cached_property
    def query(self) -> QueryResourceWithRawResponse:
        return QueryResourceWithRawResponse(self._log_explorer.query)

    @cached_property
    def datasets(self) -> DatasetsResourceWithRawResponse:
        return DatasetsResourceWithRawResponse(self._log_explorer.datasets)


class AsyncLogExplorerResourceWithRawResponse:
    def __init__(self, log_explorer: AsyncLogExplorerResource) -> None:
        self._log_explorer = log_explorer

    @cached_property
    def query(self) -> AsyncQueryResourceWithRawResponse:
        return AsyncQueryResourceWithRawResponse(self._log_explorer.query)

    @cached_property
    def datasets(self) -> AsyncDatasetsResourceWithRawResponse:
        return AsyncDatasetsResourceWithRawResponse(self._log_explorer.datasets)


class LogExplorerResourceWithStreamingResponse:
    def __init__(self, log_explorer: LogExplorerResource) -> None:
        self._log_explorer = log_explorer

    @cached_property
    def query(self) -> QueryResourceWithStreamingResponse:
        return QueryResourceWithStreamingResponse(self._log_explorer.query)

    @cached_property
    def datasets(self) -> DatasetsResourceWithStreamingResponse:
        return DatasetsResourceWithStreamingResponse(self._log_explorer.datasets)


class AsyncLogExplorerResourceWithStreamingResponse:
    def __init__(self, log_explorer: AsyncLogExplorerResource) -> None:
        self._log_explorer = log_explorer

    @cached_property
    def query(self) -> AsyncQueryResourceWithStreamingResponse:
        return AsyncQueryResourceWithStreamingResponse(self._log_explorer.query)

    @cached_property
    def datasets(self) -> AsyncDatasetsResourceWithStreamingResponse:
        return AsyncDatasetsResourceWithStreamingResponse(self._log_explorer.datasets)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["ListDeleteResponse"]


class ListDeleteResponse(BaseModel):
    id: str
    """The unique ID of the list."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .zone import (
    ZoneResource,
    AsyncZoneResource,
    ZoneResourceWithRawResponse,
    AsyncZoneResourceWithRawResponse,
    ZoneResourceWithStreamingResponse,
    AsyncZoneResourceWithStreamingResponse,
)
from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from .account import (
    AccountResource,
    AsyncAccountResource,
    AccountResourceWithRawResponse,
    AsyncAccountResourceWithRawResponse,
    AccountResourceWithStreamingResponse,
    AsyncAccountResourceWithStreamingResponse,
)

__all__ = [
    "ZoneResource",
    "AsyncZoneResource",
    "ZoneResourceWithRawResponse",
    "AsyncZoneResourceWithRawResponse",
    "ZoneResourceWithStreamingResponse",
    "AsyncZoneResourceWithStreamingResponse",
    "AccountResource",
    "AsyncAccountResource",
    "AccountResourceWithRawResponse",
    "AsyncAccountResourceWithRawResponse",
    "AccountResourceWithStreamingResponse",
    "AsyncAccountResourceWithStreamingResponse",
    "UsageResource",
    "AsyncUsageResource",
    "UsageResourceWithRawResponse",
    "AsyncUsageResourceWithRawResponse",
    "UsageResourceWithStreamingResponse",
    "AsyncUsageResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DomainGetParams"]


class DomainGetParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    domain: str

    skip_dns: bool
    """Skip DNS resolution lookups for faster response."""

    skip_ranking: bool
    """Skip the domain ranking lookup for faster responses.

    Defaults to `false` (ranking is included). Set to `true` to opt out — primarily
    used by callers like Cloudflare Radar that need to avoid a circular dependency
    when building the domain details page. Note: the bulk endpoint
    (`/intel/domain/bulk`) uses opposite defaults — see `include_ranking` there.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["SendingDomainRestrictionCreateResponse"]


class SendingDomainRestrictionCreateResponse(BaseModel):
    """
    A sending domain restriction that enforces TLS (Transport Layer Security) requirements for emails from specific domains. If TLS is required, mail without TLS from the specified domain will be dropped.
    """

    id: Optional[str] = None
    """Sending domain restriction identifier."""

    comments: Optional[str] = None

    created_at: Optional[datetime] = None

    domain: Optional[str] = None
    """Domain that requires TLS enforcement."""

    exclude: Optional[List[str]] = None
    """Excluded subdomains that are exempt from TLS requirements."""

    last_modified: Optional[datetime] = None
    """Deprecated, use `modified_at` instead. End of life: November 1, 2026."""

    modified_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ProjectListParams"]


class ProjectListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    page: int
    """Which page of projects to fetch."""

    per_page: int
    """How many projects to return per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["BulkDeleteResponse"]


class BulkDeleteResponse(BaseModel):
    id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.zaraz.configuration import Configuration

__all__ = ["ExportResource", "AsyncExportResource"]


class ExportResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ExportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ExportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ExportResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Exports full current published Zaraz configuration for a zone, secret variables
        included.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/settings/zaraz/export", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Configuration,
        )


class AsyncExportResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncExportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncExportResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Exports full current published Zaraz configuration for a zone, secret variables
        included.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/settings/zaraz/export", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Configuration,
        )


class ExportResourceWithRawResponse:
    def __init__(self, export: ExportResource) -> None:
        self._export = export

        self.get = to_raw_response_wrapper(
            export.get,
        )


class AsyncExportResourceWithRawResponse:
    def __init__(self, export: AsyncExportResource) -> None:
        self._export = export

        self.get = async_to_raw_response_wrapper(
            export.get,
        )


class ExportResourceWithStreamingResponse:
    def __init__(self, export: ExportResource) -> None:
        self._export = export

        self.get = to_streamed_response_wrapper(
            export.get,
        )


class AsyncExportResourceWithStreamingResponse:
    def __init__(self, export: AsyncExportResource) -> None:
        self._export = export

        self.get = async_to_streamed_response_wrapper(
            export.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = [
    "MaintenanceConfigGetResponse",
    "MaintenanceConfig",
    "MaintenanceConfigCompaction",
    "MaintenanceConfigSnapshotExpiration",
]


class MaintenanceConfigCompaction(BaseModel):
    """Configures compaction settings for table optimization."""

    state: Literal["enabled", "disabled"]
    """Specifies the state of maintenance operations."""

    target_size_mb: Literal["64", "128", "256", "512"]
    """Sets the target file size for compaction in megabytes. Defaults to "128"."""


class MaintenanceConfigSnapshotExpiration(BaseModel):
    """Configures snapshot expiration settings."""

    max_snapshot_age: str
    """Specifies the maximum age for snapshots.

    The system deletes snapshots older than this age. Format: <number><unit> where
    unit is d (days), h (hours), m (minutes), or s (seconds). Examples: "7d" (7
    days), "48h" (48 hours), "2880m" (2,880 minutes). Defaults to "7d".
    """

    min_snapshots_to_keep: int
    """Specifies the minimum number of snapshots to retain. Defaults to 100."""

    state: Literal["enabled", "disabled"]
    """Specifies the state of maintenance operations."""


class MaintenanceConfig(BaseModel):
    """Configures maintenance for the table."""

    compaction: Optional[MaintenanceConfigCompaction] = None
    """Configures compaction settings for table optimization."""

    snapshot_expiration: Optional[MaintenanceConfigSnapshotExpiration] = None
    """Configures snapshot expiration settings."""


class MaintenanceConfigGetResponse(BaseModel):
    """Contains table maintenance configuration."""

    maintenance_config: MaintenanceConfig
    """Configures maintenance for the table."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .inspect_get_params import InspectGetParams as InspectGetParams
from .inspect_get_response import InspectGetResponse as InspectGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.load_balancers.monitor_groups.reference_get_response import ReferenceGetResponse

__all__ = ["ReferencesResource", "AsyncReferencesResource"]


class ReferencesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ReferencesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ReferencesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReferencesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ReferencesResourceWithStreamingResponse(self)

    def get(
        self,
        monitor_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[ReferenceGetResponse]:
        """
        Get the list of resources that reference the provided monitor group.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not monitor_group_id:
            raise ValueError(f"Expected a non-empty value for `monitor_group_id` but received {monitor_group_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/load_balancers/monitor_groups/{monitor_group_id}/references",
                account_id=account_id,
                monitor_group_id=monitor_group_id,
            ),
            page=SyncSinglePage[ReferenceGetResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=ReferenceGetResponse,
        )


class AsyncReferencesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncReferencesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReferencesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReferencesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncReferencesResourceWithStreamingResponse(self)

    def get(
        self,
        monitor_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ReferenceGetResponse, AsyncSinglePage[ReferenceGetResponse]]:
        """
        Get the list of resources that reference the provided monitor group.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not monitor_group_id:
            raise ValueError(f"Expected a non-empty value for `monitor_group_id` but received {monitor_group_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/load_balancers/monitor_groups/{monitor_group_id}/references",
                account_id=account_id,
                monitor_group_id=monitor_group_id,
            ),
            page=AsyncSinglePage[ReferenceGetResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=ReferenceGetResponse,
        )


class ReferencesResourceWithRawResponse:
    def __init__(self, references: ReferencesResource) -> None:
        self._references = references

        self.get = to_raw_response_wrapper(
            references.get,
        )


class AsyncReferencesResourceWithRawResponse:
    def __init__(self, references: AsyncReferencesResource) -> None:
        self._references = references

        self.get = async_to_raw_response_wrapper(
            references.get,
        )


class ReferencesResourceWithStreamingResponse:
    def __init__(self, references: ReferencesResource) -> None:
        self._references = references

        self.get = to_streamed_response_wrapper(
            references.get,
        )


class AsyncReferencesResourceWithStreamingResponse:
    def __init__(self, references: AsyncReferencesResource) -> None:
        self._references = references

        self.get = async_to_streamed_response_wrapper(
            references.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.email_auth.spf import inspect_get_params
from ....types.email_auth.spf.inspect_get_response import InspectGetResponse

__all__ = ["InspectResource", "AsyncInspectResource"]


class InspectResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InspectResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return InspectResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InspectResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return InspectResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        zone_id: str,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[InspectGetResponse]:
        """
        Inspects a specific SPF TXT record and returns a parsed tree structure in the
        spflimit-worker format.

        The record ID must be provided via the `id` query parameter.

        Returns a recursive tree showing:

        - Parsed components with their qualifiers and types
        - Nested includes recursively resolved within components
        - Per-component and total lookup counts
        - Detailed error information with context

        Args:
          zone_id: Identifier.

          id: DNS record ID (rec_tag) to inspect

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/email/auth/spf/inspect", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"id": id}, inspect_get_params.InspectGetParams),
                post_parser=ResultWrapper[Optional[InspectGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[InspectGetResponse]], ResultWrapper[InspectGetResponse]),
        )


class AsyncInspectResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInspectResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInspectResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInspectResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncInspectResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        zone_id: str,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[InspectGetResponse]:
        """
        Inspects a specific SPF TXT record and returns a parsed tree structure in the
        spflimit-worker format.

        The record ID must be provided via the `id` query parameter.

        Returns a recursive tree showing:

        - Parsed components with their qualifiers and types
        - Nested includes recursively resolved within components
        - Per-component and total lookup counts
        - Detailed error information with context

        Args:
          zone_id: Identifier.

          id: DNS record ID (rec_tag) to inspect

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/email/auth/spf/inspect", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"id": id}, inspect_get_params.InspectGetParams),
                post_parser=ResultWrapper[Optional[InspectGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[InspectGetResponse]], ResultWrapper[InspectGetResponse]),
        )


class InspectResourceWithRawResponse:
    def __init__(self, inspect: InspectResource) -> None:
        self._inspect = inspect

        self.get = to_raw_response_wrapper(
            inspect.get,
        )


class AsyncInspectResourceWithRawResponse:
    def __init__(self, inspect: AsyncInspectResource) -> None:
        self._inspect = inspect

        self.get = async_to_raw_response_wrapper(
            inspect.get,
        )


class InspectResourceWithStreamingResponse:
    def __init__(self, inspect: InspectResource) -> None:
        self._inspect = inspect

        self.get = to_streamed_response_wrapper(
            inspect.get,
        )


class AsyncInspectResourceWithStreamingResponse:
    def __init__(self, inspect: AsyncInspectResource) -> None:
        self._inspect = inspect

        self.get = async_to_streamed_response_wrapper(
            inspect.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date, datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["AuditHistoryParams"]


class AuditHistoryParams(TypedDict, total=False):
    account_id: Required[str]
    """The unique ID that identifies the account."""

    action_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC3339 timestamp of the source audit log entry's action time.

    Used to narrow the source-entry lookup window. Provide the `action.time` value
    from the audit log identified by `id`.
    """

    before: Required[Annotated[Union[str, date], PropertyInfo(format="iso8601")]]
    """Limits the returned results to logs older than the specified date.

    This can be a date string 2019-04-30 (interpreted in UTC) or an absolute
    timestamp that conforms to RFC3339.
    """

    since: Required[Annotated[Union[str, date], PropertyInfo(format="iso8601")]]
    """Limits the returned results to logs newer than the specified date.

    This can be a date string 2019-04-30 (interpreted in UTC) or an absolute
    timestamp that conforms to RFC3339.
    """

    cursor: str
    """The cursor is an opaque token used to paginate through large sets of records.

    It indicates the position from which to continue when requesting the next set of
    records. A valid cursor value can be obtained from the cursor object in the
    result_info structure of a previous response.
    """

    direction: Literal["desc", "asc"]
    """Sets sorting order."""

    limit: float
    """The number limits the objects to return.

    The cursor attribute may be used to iterate over the next batch of objects if
    there are more than the limit.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["WorkflowGetResponse", "Instances", "Schedule"]


class Instances(BaseModel):
    complete: Optional[float] = None

    errored: Optional[float] = None

    paused: Optional[float] = None

    queued: Optional[float] = None

    rolling_back: Optional[float] = FieldInfo(alias="rollingBack", default=None)

    running: Optional[float] = None

    terminated: Optional[float] = None

    waiting: Optional[float] = None

    waiting_for_pause: Optional[float] = FieldInfo(alias="waitingForPause", default=None)


class Schedule(BaseModel):
    cron: str

    next_instance: str


class WorkflowGetResponse(BaseModel):
    id: str

    class_name: str

    created_on: datetime

    instances: Instances

    modified_on: datetime

    name: str

    script_name: str

    triggered_on: Optional[datetime] = None

    schedules: Optional[List[Schedule]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["RecordingStartTrackRecordingParams", "Layers"]


class RecordingStartTrackRecordingParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    meeting_id: Required[str]
    """ID of the meeting to record."""

    layers: Dict[str, Layers]
    """Optional audio layer configuration.

    If omitted, RealtimeKit records all participant audio using the default file
    name prefix.
    """

    user_ids: SequenceNotStr[str]
    """Optional list of participant user IDs to record.

    Selective track recording (`user_ids`) is in early beta contact support to use
    this feature.
    """


class Layers(TypedDict, total=False):
    file_name_prefix: str
    """A file name prefix to apply for files generated from this layer"""

    media_kind: Literal["audio"]
    """Media kind to record. Track recording currently supports audio only."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["OAuthClientUpdateResponse", "ClientURIVerification"]


class ClientURIVerification(BaseModel):
    """Client URI domain control verification state."""

    status: Optional[Literal["pending", "in_progress", "verified", "failed"]] = None
    """Current verification status for the client URI host."""

    text: Optional[str] = None
    """
    Exact TXT record value that must be added to DNS to prove ownership of the
    client URI host.
    """


class OAuthClientUpdateResponse(BaseModel):
    """Fields shared by OAuth client responses and create/update requests."""

    client_id: str
    """The unique identifier for an OAuth client."""

    visibility: Literal["public", "private"]
    """Visibility of the OAuth client."""

    allowed_cors_origins: Optional[List[str]] = None
    """Array of allowed CORS origins."""

    client_name: Optional[str] = None
    """Human-readable name of the OAuth client."""

    client_uri: Optional[str] = None
    """URL of the home page of the client."""

    client_uri_verification: Optional[ClientURIVerification] = None
    """Client URI domain control verification state."""

    created_at: Optional[datetime] = None
    """Timestamp when the OAuth client was created."""

    grant_types: Optional[List[Literal["authorization_code", "refresh_token"]]] = None
    """Array of OAuth grant types the client is allowed to use.

    `authorization_code` is required; `refresh_token` may be included optionally.
    """

    has_rotated_secret: Optional[bool] = None
    """
    Indicates whether the client has a rotated secret that has not yet been deleted.
    """

    logo_uri: Optional[str] = None
    """URL of the client's logo."""

    policy_uri: Optional[str] = None
    """URL that points to a privacy policy document."""

    post_logout_redirect_uris: Optional[List[str]] = None
    """Array of allowed post-logout redirect URIs."""

    promoted_at: Optional[datetime] = None
    """Timestamp when the OAuth client was promoted to public visibility."""

    redirect_uris: Optional[List[str]] = None
    """Array of allowed redirect URIs for the client."""

    response_types: Optional[List[Literal["token", "id_token", "code"]]] = None
    """Array of OAuth response types the client is allowed to use."""

    scopes: Optional[List[str]] = None
    """Array of OAuth scopes the client is allowed to request.

    Colon-delimited scopes are not accepted. Dot-delimited scopes are validated
    against available OAuth API scopes; simple identity scopes are allowed. Protocol
    scopes `offline_access` and `openid` are added or removed automatically based on
    `grant_types` and `response_types`.
    """

    token_endpoint_auth_method: Optional[Literal["none", "client_secret_basic", "client_secret_post"]] = None
    """The authentication method the client uses at the token endpoint."""

    tos_uri: Optional[str] = None
    """URL that points to a terms of service document."""

    updated_at: Optional[datetime] = None
    """Timestamp when the OAuth client was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.zones.ct import alerting_edit_params
from ....types.zones.ct.alerting_get_response import AlertingGetResponse
from ....types.zones.ct.alerting_edit_response import AlertingEditResponse

__all__ = ["AlertingResource", "AsyncAlertingResource"]


class AlertingResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AlertingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AlertingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AlertingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AlertingResourceWithStreamingResponse(self)

    def edit(
        self,
        *,
        zone_id: str,
        enabled: bool,
        emails: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AlertingEditResponse]:
        """
        Create or update the Certificate Transparency alerting subscription for a zone.
        Enables or disables email notifications when certificates are issued for the
        zone's domains. For Free and Pro zones, the subscription is toggled on or off
        using the enabled field. Notification emails are sent to all users with SSL
        permissions on the zone. For Business and Enterprise zones, the emails field is
        required and controls which addresses receive alerts. Setting emails to an empty
        list disables the subscription regardless of the enabled field. A maximum of 10
        email addresses may be configured.

        Args:
          zone_id: Identifier.

          enabled: Whether CT alerting is enabled for the zone.

          emails: Email addresses that receive CT alert notifications. Only present and
              configurable for Business and Enterprise zones. Maximum of 10 addresses. For
              Free and Pro zones, notifications are sent to all users with SSL permissions on
              the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._patch(
            path_template("/zones/{zone_id}/ct/alerting", zone_id=zone_id),
            body=maybe_transform(
                {
                    "enabled": enabled,
                    "emails": emails,
                },
                alerting_edit_params.AlertingEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AlertingEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AlertingEditResponse]], ResultWrapper[AlertingEditResponse]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AlertingGetResponse]:
        """
        Retrieve the Certificate Transparency alerting subscription settings for a zone.
        Returns whether CT monitoring is enabled and, for Business and Enterprise zones,
        the list of email addresses that receive alerts.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/ct/alerting", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AlertingGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AlertingGetResponse]], ResultWrapper[AlertingGetResponse]),
        )


class AsyncAlertingResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAlertingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAlertingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAlertingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAlertingResourceWithStreamingResponse(self)

    async def edit(
        self,
        *,
        zone_id: str,
        enabled: bool,
        emails: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AlertingEditResponse]:
        """
        Create or update the Certificate Transparency alerting subscription for a zone.
        Enables or disables email notifications when certificates are issued for the
        zone's domains. For Free and Pro zones, the subscription is toggled on or off
        using the enabled field. Notification emails are sent to all users with SSL
        permissions on the zone. For Business and Enterprise zones, the emails field is
        required and controls which addresses receive alerts. Setting emails to an empty
        list disables the subscription regardless of the enabled field. A maximum of 10
        email addresses may be configured.

        Args:
          zone_id: Identifier.

          enabled: Whether CT alerting is enabled for the zone.

          emails: Email addresses that receive CT alert notifications. Only present and
              configurable for Business and Enterprise zones. Maximum of 10 addresses. For
              Free and Pro zones, notifications are sent to all users with SSL permissions on
              the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._patch(
            path_template("/zones/{zone_id}/ct/alerting", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "enabled": enabled,
                    "emails": emails,
                },
                alerting_edit_params.AlertingEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AlertingEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AlertingEditResponse]], ResultWrapper[AlertingEditResponse]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AlertingGetResponse]:
        """
        Retrieve the Certificate Transparency alerting subscription settings for a zone.
        Returns whether CT monitoring is enabled and, for Business and Enterprise zones,
        the list of email addresses that receive alerts.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/ct/alerting", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AlertingGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AlertingGetResponse]], ResultWrapper[AlertingGetResponse]),
        )


class AlertingResourceWithRawResponse:
    def __init__(self, alerting: AlertingResource) -> None:
        self._alerting = alerting

        self.edit = to_raw_response_wrapper(
            alerting.edit,
        )
        self.get = to_raw_response_wrapper(
            alerting.get,
        )


class AsyncAlertingResourceWithRawResponse:
    def __init__(self, alerting: AsyncAlertingResource) -> None:
        self._alerting = alerting

        self.edit = async_to_raw_response_wrapper(
            alerting.edit,
        )
        self.get = async_to_raw_response_wrapper(
            alerting.get,
        )


class AlertingResourceWithStreamingResponse:
    def __init__(self, alerting: AlertingResource) -> None:
        self._alerting = alerting

        self.edit = to_streamed_response_wrapper(
            alerting.edit,
        )
        self.get = to_streamed_response_wrapper(
            alerting.get,
        )


class AsyncAlertingResourceWithStreamingResponse:
    def __init__(self, alerting: AsyncAlertingResource) -> None:
        self._alerting = alerting

        self.edit = async_to_streamed_response_wrapper(
            alerting.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            alerting.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .level_get_response import LevelGetResponse as LevelGetResponse
from .level_create_params import LevelCreateParams as LevelCreateParams
from .level_list_response import LevelListResponse as LevelListResponse
from .level_update_params import LevelUpdateParams as LevelUpdateParams
from .level_create_response import LevelCreateResponse as LevelCreateResponse
from .level_update_response import LevelUpdateResponse as LevelUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ......_models import BaseModel

__all__ = ["ServerReadResponse", "ErrorDetails", "UpdatedPrompt", "UpdatedTool"]


class ErrorDetails(BaseModel):
    cause: Optional[str] = None
    """Underlying error message"""

    is_upstream: Optional[bool] = None
    """True = MCP server returned an error. False = couldn't reach the server"""

    mcp_code: Optional[float] = None
    """MCP protocol error code"""

    retryable: Optional[bool] = None
    """Whether the error is transient and worth retrying"""

    status_code: Optional[float] = None
    """HTTP status code from the server"""


class UpdatedPrompt(BaseModel):
    name: str

    alias: Optional[str] = None

    description: Optional[str] = None

    enabled: Optional[bool] = None


class UpdatedTool(BaseModel):
    name: str

    alias: Optional[str] = None

    description: Optional[str] = None

    enabled: Optional[bool] = None


class ServerReadResponse(BaseModel):
    id: str
    """server id"""

    auth_type: Literal["oauth", "bearer", "unauthenticated"]

    hostname: str

    name: str

    prompts: List[Dict[str, object]]

    tools: List[Dict[str, object]]

    created_at: Optional[datetime] = None

    created_by: Optional[str] = None

    description: Optional[str] = None

    error: Optional[str] = None

    error_details: Optional[ErrorDetails] = None

    is_shared_oauth_callback_enabled: Optional[bool] = None
    """
    When true, the gateway worker uses the shared Cloudflare-owned OAuth callback
    endpoint as the redirect_uri for upstream on-behalf OAuth, instead of the
    customer portal hostname. Defaults to false (off); opt in per server by setting
    true. Effective behavior is gated by the gateway worker's per-env rollout mode
    KV key.
    """

    last_successful_sync: Optional[datetime] = None

    last_synced: Optional[datetime] = None

    modified_at: Optional[datetime] = None

    modified_by: Optional[str] = None

    secure_web_gateway: Optional[bool] = None
    """Route outbound traffic to this MCP server through Zero Trust Secure Web Gateway"""

    status: Optional[str] = None

    updated_prompts: Optional[List[UpdatedPrompt]] = None

    updated_tools: Optional[List[UpdatedTool]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ......_types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......_base_client import make_request_options
from ......types.zero_trust.dlp.sensitivity_groups.levels import order_update_params
from ......types.zero_trust.dlp.sensitivity_groups.levels.order_get_response import OrderGetResponse
from ......types.zero_trust.dlp.sensitivity_groups.levels.order_update_response import OrderUpdateResponse

__all__ = ["OrderResource", "AsyncOrderResource"]


class OrderResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OrderResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OrderResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OrderResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OrderResourceWithStreamingResponse(self)

    def update(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        level_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OrderUpdateResponse]:
        """
        Set the ordering of levels within a sensitivity group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return self._put(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/level_order",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            body=maybe_transform({"level_ids": level_ids}, order_update_params.OrderUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OrderUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OrderUpdateResponse]], ResultWrapper[OrderUpdateResponse]),
        )

    def get(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OrderGetResponse]:
        """
        Retrieve the ordered list of level IDs for a sensitivity group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return self._get(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/level_order",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OrderGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OrderGetResponse]], ResultWrapper[OrderGetResponse]),
        )


class AsyncOrderResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOrderResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOrderResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOrderResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOrderResourceWithStreamingResponse(self)

    async def update(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        level_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OrderUpdateResponse]:
        """
        Set the ordering of levels within a sensitivity group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return await self._put(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/level_order",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            body=await async_maybe_transform({"level_ids": level_ids}, order_update_params.OrderUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OrderUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OrderUpdateResponse]], ResultWrapper[OrderUpdateResponse]),
        )

    async def get(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OrderGetResponse]:
        """
        Retrieve the ordered list of level IDs for a sensitivity group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return await self._get(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/level_order",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OrderGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OrderGetResponse]], ResultWrapper[OrderGetResponse]),
        )


class OrderResourceWithRawResponse:
    def __init__(self, order: OrderResource) -> None:
        self._order = order

        self.update = to_raw_response_wrapper(
            order.update,
        )
        self.get = to_raw_response_wrapper(
            order.get,
        )


class AsyncOrderResourceWithRawResponse:
    def __init__(self, order: AsyncOrderResource) -> None:
        self._order = order

        self.update = async_to_raw_response_wrapper(
            order.update,
        )
        self.get = async_to_raw_response_wrapper(
            order.get,
        )


class OrderResourceWithStreamingResponse:
    def __init__(self, order: OrderResource) -> None:
        self._order = order

        self.update = to_streamed_response_wrapper(
            order.update,
        )
        self.get = to_streamed_response_wrapper(
            order.get,
        )


class AsyncOrderResourceWithStreamingResponse:
    def __init__(self, order: AsyncOrderResource) -> None:
        self._order = order

        self.update = async_to_streamed_response_wrapper(
            order.update,
        )
        self.get = async_to_streamed_response_wrapper(
            order.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .isps import (
    ISPsResource,
    AsyncISPsResource,
    ISPsResourceWithRawResponse,
    AsyncISPsResourceWithRawResponse,
    ISPsResourceWithStreamingResponse,
    AsyncISPsResourceWithStreamingResponse,
)
from .devices import (
    DevicesResource,
    AsyncDevicesResource,
    DevicesResourceWithRawResponse,
    AsyncDevicesResourceWithRawResponse,
    DevicesResourceWithStreamingResponse,
    AsyncDevicesResourceWithStreamingResponse,
)

__all__ = [
    "ISPsResource",
    "AsyncISPsResource",
    "ISPsResourceWithRawResponse",
    "AsyncISPsResourceWithRawResponse",
    "ISPsResourceWithStreamingResponse",
    "AsyncISPsResourceWithStreamingResponse",
    "DevicesResource",
    "AsyncDevicesResource",
    "DevicesResourceWithRawResponse",
    "AsyncDevicesResourceWithRawResponse",
    "DevicesResourceWithStreamingResponse",
    "AsyncDevicesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["IntegrationGetResponse", "AuthorizationLink"]


class AuthorizationLink(BaseModel):
    """Authorization link for the integration."""

    components: Optional[Dict[str, object]] = None

    link: Optional[str] = None


class IntegrationGetResponse(BaseModel):
    """Serializer for v2 integration detail response with use cases."""

    id: str
    """Integration ID."""

    application: Dict[str, Optional[str]]

    auth_method: Optional[Dict[str, str]] = None
    """The integration's authentication method."""

    authorization_link: Optional[AuthorizationLink] = None
    """Authorization link for the integration."""

    created: datetime
    """When the integration was created."""

    credentials_expiry: datetime
    """Credentials expiry time."""

    dlp_profiles: List[str]
    """DLP Profiles enabled for the integration."""

    health_details: List[Dict[str, object]]
    """Health details with remediation hints."""

    is_paused: bool
    """Whether the user paused the integration."""

    last_hydrated: datetime
    """Last time the integration was hydrated."""

    name: str
    """Name of the integration."""

    organization_id: int
    """Organization ID."""

    status: str
    """Integration status."""

    updated: datetime
    """When the integration was last updated."""

    use_cases: List[Dict[str, object]]
    """Use cases enabled for the integration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.radar.as112 import top_edns_params, top_dnssec_params, top_locations_params, top_ip_version_params
from ....types.radar.as112.top_edns_response import TopEdnsResponse
from ....types.radar.as112.top_dnssec_response import TopDNSSECResponse
from ....types.radar.as112.top_locations_response import TopLocationsResponse
from ....types.radar.as112.top_ip_version_response import TopIPVersionResponse

__all__ = ["TopResource", "AsyncTopResource"]


class TopResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TopResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TopResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TopResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TopResourceWithStreamingResponse(self)

    def dnssec(
        self,
        dnssec: Literal["SUPPORTED", "NOT_SUPPORTED"],
        *,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopDNSSECResponse:
        """
        Retrieves the top locations of DNS queries to AS112 with DNSSEC (DNS Security
        Extensions) support.

        Args:
          dnssec: DNSSEC (DNS Security Extensions) status.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dnssec:
            raise ValueError(f"Expected a non-empty value for `dnssec` but received {dnssec!r}")
        return self._get(
            path_template("/radar/as112/top/locations/dnssec/{dnssec}", dnssec=dnssec),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "name": name,
                    },
                    top_dnssec_params.TopDNSSECParams,
                ),
                post_parser=ResultWrapper[TopDNSSECResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopDNSSECResponse], ResultWrapper[TopDNSSECResponse]),
        )

    def edns(
        self,
        edns: Literal["SUPPORTED", "NOT_SUPPORTED"],
        *,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopEdnsResponse:
        """
        Retrieves the top locations of DNS queries to AS112 with EDNS (Extension
        Mechanisms for DNS) support.

        Args:
          edns: EDNS (Extension Mechanisms for DNS) status.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not edns:
            raise ValueError(f"Expected a non-empty value for `edns` but received {edns!r}")
        return self._get(
            path_template("/radar/as112/top/locations/edns/{edns}", edns=edns),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "name": name,
                    },
                    top_edns_params.TopEdnsParams,
                ),
                post_parser=ResultWrapper[TopEdnsResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopEdnsResponse], ResultWrapper[TopEdnsResponse]),
        )

    def ip_version(
        self,
        ip_version: Literal["IPv4", "IPv6"],
        *,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopIPVersionResponse:
        """
        Retrieves the top locations of DNS queries to AS112 for an IP version.

        Args:
          ip_version: IP version.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ip_version:
            raise ValueError(f"Expected a non-empty value for `ip_version` but received {ip_version!r}")
        return self._get(
            path_template("/radar/as112/top/locations/ip_version/{ip_version}", ip_version=ip_version),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "name": name,
                    },
                    top_ip_version_params.TopIPVersionParams,
                ),
                post_parser=ResultWrapper[TopIPVersionResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopIPVersionResponse], ResultWrapper[TopIPVersionResponse]),
        )

    def locations(
        self,
        *,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopLocationsResponse:
        """
        Retrieves the top locations by AS112 DNS queries.

        Args:
          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/as112/top/locations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "name": name,
                    },
                    top_locations_params.TopLocationsParams,
                ),
                post_parser=ResultWrapper[TopLocationsResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopLocationsResponse], ResultWrapper[TopLocationsResponse]),
        )


class AsyncTopResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTopResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTopResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTopResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTopResourceWithStreamingResponse(self)

    async def dnssec(
        self,
        dnssec: Literal["SUPPORTED", "NOT_SUPPORTED"],
        *,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopDNSSECResponse:
        """
        Retrieves the top locations of DNS queries to AS112 with DNSSEC (DNS Security
        Extensions) support.

        Args:
          dnssec: DNSSEC (DNS Security Extensions) status.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dnssec:
            raise ValueError(f"Expected a non-empty value for `dnssec` but received {dnssec!r}")
        return await self._get(
            path_template("/radar/as112/top/locations/dnssec/{dnssec}", dnssec=dnssec),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "name": name,
                    },
                    top_dnssec_params.TopDNSSECParams,
                ),
                post_parser=ResultWrapper[TopDNSSECResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopDNSSECResponse], ResultWrapper[TopDNSSECResponse]),
        )

    async def edns(
        self,
        edns: Literal["SUPPORTED", "NOT_SUPPORTED"],
        *,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopEdnsResponse:
        """
        Retrieves the top locations of DNS queries to AS112 with EDNS (Extension
        Mechanisms for DNS) support.

        Args:
          edns: EDNS (Extension Mechanisms for DNS) status.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not edns:
            raise ValueError(f"Expected a non-empty value for `edns` but received {edns!r}")
        return await self._get(
            path_template("/radar/as112/top/locations/edns/{edns}", edns=edns),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "name": name,
                    },
                    top_edns_params.TopEdnsParams,
                ),
                post_parser=ResultWrapper[TopEdnsResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopEdnsResponse], ResultWrapper[TopEdnsResponse]),
        )

    async def ip_version(
        self,
        ip_version: Literal["IPv4", "IPv6"],
        *,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopIPVersionResponse:
        """
        Retrieves the top locations of DNS queries to AS112 for an IP version.

        Args:
          ip_version: IP version.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ip_version:
            raise ValueError(f"Expected a non-empty value for `ip_version` but received {ip_version!r}")
        return await self._get(
            path_template("/radar/as112/top/locations/ip_version/{ip_version}", ip_version=ip_version),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "name": name,
                    },
                    top_ip_version_params.TopIPVersionParams,
                ),
                post_parser=ResultWrapper[TopIPVersionResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopIPVersionResponse], ResultWrapper[TopIPVersionResponse]),
        )

    async def locations(
        self,
        *,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopLocationsResponse:
        """
        Retrieves the top locations by AS112 DNS queries.

        Args:
          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/as112/top/locations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "name": name,
                    },
                    top_locations_params.TopLocationsParams,
                ),
                post_parser=ResultWrapper[TopLocationsResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopLocationsResponse], ResultWrapper[TopLocationsResponse]),
        )


class TopResourceWithRawResponse:
    def __init__(self, top: TopResource) -> None:
        self._top = top

        self.dnssec = to_raw_response_wrapper(
            top.dnssec,
        )
        self.edns = to_raw_response_wrapper(
            top.edns,
        )
        self.ip_version = to_raw_response_wrapper(
            top.ip_version,
        )
        self.locations = to_raw_response_wrapper(
            top.locations,
        )


class AsyncTopResourceWithRawResponse:
    def __init__(self, top: AsyncTopResource) -> None:
        self._top = top

        self.dnssec = async_to_raw_response_wrapper(
            top.dnssec,
        )
        self.edns = async_to_raw_response_wrapper(
            top.edns,
        )
        self.ip_version = async_to_raw_response_wrapper(
            top.ip_version,
        )
        self.locations = async_to_raw_response_wrapper(
            top.locations,
        )


class TopResourceWithStreamingResponse:
    def __init__(self, top: TopResource) -> None:
        self._top = top

        self.dnssec = to_streamed_response_wrapper(
            top.dnssec,
        )
        self.edns = to_streamed_response_wrapper(
            top.edns,
        )
        self.ip_version = to_streamed_response_wrapper(
            top.ip_version,
        )
        self.locations = to_streamed_response_wrapper(
            top.locations,
        )


class AsyncTopResourceWithStreamingResponse:
    def __init__(self, top: AsyncTopResource) -> None:
        self._top = top

        self.dnssec = async_to_streamed_response_wrapper(
            top.dnssec,
        )
        self.edns = async_to_streamed_response_wrapper(
            top.edns,
        )
        self.ip_version = async_to_streamed_response_wrapper(
            top.ip_version,
        )
        self.locations = async_to_streamed_response_wrapper(
            top.locations,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["FieldGetResponse"]


class FieldGetResponse(BaseModel):
    key: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["DatasetSummary"]


class DatasetSummary(BaseModel):
    """A Log Explorer dataset summary.

    List endpoints return this type and omit
    field configuration; use the single-dataset endpoint to retrieve it.
    """

    created_at: datetime
    """RFC3339 timestamp recording when the API created this dataset."""

    dataset: str
    """Dataset type name (e.g. `http_requests`)."""

    dataset_id: str
    """Unique dataset ID."""

    enabled: bool
    """Whether log ingest is currently active for this dataset."""

    object_id: str
    """Public ID of the account or zone that owns this dataset."""

    object_type: Literal["account", "zone"]
    """Whether this dataset belongs to an account or a zone."""

    updated_at: datetime
    """RFC3339 timestamp recording when the API last updated this dataset."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel
from .available_dataset import AvailableDataset
from ....shared.response_info import ResponseInfo

__all__ = ["AvailableList"]


class AvailableList(BaseModel):
    errors: List[ResponseInfo]

    messages: List[str]

    success: bool

    result: Optional[List[AvailableDataset]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["ISPListParams"]


class ISPListParams(TypedDict, total=False):
    account_id: Required[str]
    """Unique identifier linked to an account."""

    per_page: Required[int]
    """Number of items per page"""

    cursor: str
    """Cursor for cursor-based pagination. Mutually exclusive with page."""

    from_: Annotated[Union[str, datetime], PropertyInfo(alias="from", format="iso8601")]
    """Start time for the query in ISO 8601 format."""

    page: int
    """Page number of paginated results. Mutually exclusive with cursor."""

    sort_by: Literal["time_start"]
    """The field to sort results by."""

    sort_order: Literal["ASC", "DESC"]
    """The order to sort results."""

    to: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """End time for the query in ISO 8601 format."""
